"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_decorators_1 = require("core-decorators");
const assert = require("assert");
let ContextCommands = class ContextCommands {
    constructor() {
        this.commands = new Map();
    }
    list() {
        const commands = Array.from(this.commands.keys());
        process.stdout.write('\nAvailable Commands\n');
        process.stdout.write('invoke via :<command> [--arg1, --arg2, ...] or [--arg=value]\n');
        process.stdout.write('see command options below for accepted arguments\n\n');
        commands.forEach(name => {
            const command = this.commands.get(name);
            process.stdout.write(`${name} -- ${command.description}\n`);
            if (command.options) {
                process.stdout.write(`   options: ${command.options.join(' ')}\n`);
            }
        });
    }
    register(command, contextCommand) {
        this.ensureUniqueCommand(command);
        ensureCommandFormatting(command);
        this.commands.set(command, contextCommand);
    }
    listen() {
        process.stdin.on('data', (data) => {
            try {
                this.processInput(data);
            }
            catch (e) {
                process.stderr.write(`Error invoking command, ${e.message}`);
            }
        });
    }
    processInput(buffer) {
        const input = buffer.toString().trim();
        if (isCommand(input)) {
            const sArgs = input.split(/\s+/);
            const command = sArgs.shift().replace(/^\:/, '');
            const args = parseArgs(sArgs);
            this.invoke(command, args);
        }
    }
    invoke(command, args) {
        const cmd = this.commands.get(command);
        if (cmd) {
            cmd.handler(args);
        }
    }
    ensureUniqueCommand(command) {
        assert(!this.commands.has(command), `Command ${command} already registered. Pick a unique command name`);
    }
};
ContextCommands = __decorate([
    core_decorators_1.autobind
], ContextCommands);
exports.ContextCommands = ContextCommands;
function isCommand(input) {
    return /^\:/.test(input);
}
function parseArgs(vArgs) {
    return vArgs.reduce((acc, arg) => {
        arg = arg.replace(/^\-+/, '');
        if (/\=/.test(arg)) {
            const kv = arg.split('=');
            acc[kv[0]] = kv[1];
        }
        else {
            acc[arg] = true;
        }
        return acc;
    }, {});
}
function ensureCommandFormatting(command) {
    assert(/^\w/.test(command), `Command should start with a letter, check ${command}`);
    assert(!(/[^\w-]/.test(command)), `Command should only contain [a-zA-z0-9_-], check ${command}`);
    assert(!(/\s/).test(command), `Commands should not have spaces, check ${command}`);
}
