"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const taskContext_1 = require("./taskContext");
const DefaultRegistry = require("undertaker-registry");
const clean_1 = require("./tasks/clean");
const statics_1 = require("./tasks/statics");
const tsLint_1 = require("./tasks/tsLint");
const compiler_1 = require("./tasks/compiler");
const serverTest_1 = require("./tasks/serverTest");
const server_1 = require("./tasks/server");
const webpackDevServer_1 = require("./tasks/webpackDevServer");
class Registry extends DefaultRegistry {
    constructor(options) {
        super();
        assert(typeof options.rootPath === 'string', 'rootPath must be defined in Registry options');
        this.context = new taskContext_1.TaskContext(options);
    }
    init(gulp) {
        super.init(gulp);
        const { task, parallel, series } = gulp;
        const server = new server_1.Server(this.context);
        const cleaner = new clean_1.Cleaner(this.context);
        const statics = new statics_1.Statics(this.context);
        const tslint = new tsLint_1.TsLinter(this.context);
        const compiler = new compiler_1.ServerCompiler(this.context);
        const serverTest = new serverTest_1.ServerTest(this.context, server);
        const webpackDevServer = new webpackDevServer_1.WebpackDevServer(this.context);
        task('clean:client', cleaner.cleanClient);
        task('clean:server', cleaner.cleanServer);
        task('clean', parallel('clean:client', 'clean:server'));
        task('statics:build', statics.build);
        task('tslint:tasks', tslint.lintTasks);
        task('tslint:server', tslint.lintServer);
        task('tslint:client', tslint.lintClient);
        task('tslint', parallel('tslint:client', 'tslint:client'));
        task('client:prebuild', parallel('tslint:client', series('clean:client', 'statics:build')));
        task('client:build', series('client:prebuild' /* webpack here */));
        task('client:devServer', series('client:prebuild', webpackDevServer.serve));
        task('server:compile', compiler.compile);
        task('server:test:single', series('server:compile', serverTest.single));
        task('server:test', series(parallel('server:test:single', compiler.watch), serverTest.continuous));
        task('server:run', parallel('tslint:server', parallel(compiler.watch, serverTest.applyHooks), series('clean:server', 'server:compile', server.serve)));
        task('dev', parallel('client:devServer', 'server:run'));
        task('default', task('dev'));
    }
}
exports.Registry = Registry;
