"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_decorators_1 = require("core-decorators");
const killTree = require("tree-kill");
const contextCommands_1 = require("./contextCommands");
let TaskContext = class TaskContext {
    constructor(config) {
        this.childProcesses = [];
        this.teardowns = [];
        this.commands = new contextCommands_1.ContextCommands();
        this.registerCommand = this.commands.register;
        this.config = mergeDefaultConfig(config);
        this.commands.register('q', {
            handler: this.exit,
            description: 'Quit all tasks'
        });
        this.commands.register('commands', {
            handler: this.commands.list,
            description: 'List available commands'
        });
        this.commands.listen();
        process.on('exit', this.exitGracefully);
    }
    registerChildProcess(child, options = {}) {
        if (!options.silent) {
            child.stdout.pipe(process.stdout);
            child.stderr.pipe(process.stderr);
        }
        this.childProcesses.push({ process: child, options });
    }
    onExit(cb) {
        this.teardowns.push(cb);
    }
    exit() {
        process.stdout.write('exiting\n');
        this.exitGracefully();
        process.exit(0);
    }
    exitGracefully() {
        this.childProcesses.forEach(child => {
            killTree(child.process.pid);
            child.process.kill();
        });
        this.teardowns.forEach(cb => cb && cb());
    }
};
TaskContext = __decorate([
    core_decorators_1.autobind
], TaskContext);
exports.TaskContext = TaskContext;
function mergeDefaultConfig(config) {
    return Object.assign({ buildDir: (process.env.NODE_ENV === 'production' ? 'dist' : 'build') }, config);
}
