"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_decorators_1 = require("core-decorators");
const gulp = require("gulp");
const mocha = require("gulp-mocha");
const debug = require("gulp-debug");
const lodash_1 = require("lodash");
const taskGroup_1 = require("../taskGroup");
let ServerTest = class ServerTest extends taskGroup_1.TaskGroup {
    constructor(context, server) {
        super(context);
        this.server = server;
        const { rootPath, buildDir } = context.config;
        this.specFilesGlob = `${rootPath}/${buildDir}/**/*.spec.js`;
    }
    single() {
        return gulp
            .src(this.specFilesGlob)
            .pipe(debug({ title: 'Server test files:' }))
            .pipe(mocha());
    }
    continuous(done) {
        const { onExit } = this.context;
        onExit(done);
        this.single();
        this.watcher = gulp.watch(this.specFilesGlob, this.single);
        done();
        return this.watcher;
    }
    applyHooks(done) {
        this.context.registerCommand('serverTest', {
            handler: (args) => {
                if (args.start) {
                    this.startServerTests();
                }
                else if (args.stop) {
                    this.stopServerTests();
                }
            },
            description: 'Start running server specs, re-run as server recompiles',
            options: ['--start', '--stop']
        });
        done();
    }
    startServerTests() {
        if (this.server.running && !this.watcher) {
            this.continuous(lodash_1.noop);
        }
        else if (!this.server.running) {
            logHookFailure();
        }
    }
    stopServerTests() {
        if (this.server.running && this.watcher) {
            process.stdout.write('stopping server tests\n');
            this.watcher.close();
            this.watcher = null;
        }
        else if (!this.server.running) {
            logHookFailure();
        }
    }
};
ServerTest = __decorate([
    core_decorators_1.autobind
], ServerTest);
exports.ServerTest = ServerTest;
function logHookFailure() {
    process.stdout.write('No running server, cannot hook server specs\n');
}
